@@ -1,5 +1,5 @@
 # Import all subpackages
-import dill  # dill before pickle
+import dill
 import vipy.show  # matplotlib first
 import vipy.activity
 import vipy.annotation
